/*
 * Copyright (c) 2006-2014 Rogério Liesenfeld
 * This file is subject to the terms of the MIT license (see LICENSE.txt).
 */
package mockit.internal.mockups;

import java.lang.reflect.Type;

import org.jetbrains.annotations.*;

import mockit.*;
import mockit.external.asm.*;
import mockit.internal.*;
import mockit.internal.capturing.*;
import mockit.internal.util.*;

public final class CaptureOfMockedUpImplementations extends CaptureOfImplementations<Void>
{
   private final MockClassSetup mockClassSetup;

   public CaptureOfMockedUpImplementations(MockUp<?> mockUp, Type baseType)
   {
      Class<?> baseClassType = Utilities.getClassType(baseType);
      mockClassSetup = new MockClassSetup(baseClassType, baseType, mockUp, null);
   }

   @NotNull @Override
   protected BaseClassModifier createModifier(
      @Nullable ClassLoader cl, @NotNull ClassReader cr, @NotNull Class<?> baseType, Void typeMetadata)
   {
      return mockClassSetup.createClassModifier(cr);
   }

   @Override
   protected void redefineClass(@NotNull Class<?> realClass, @NotNull byte[] modifiedClass)
   {
      mockClassSetup.applyClassModifications(realClass, modifiedClass);
   }

   @Nullable
   public <T> Class<T> apply()
   {
      @SuppressWarnings("unchecked") Class<T> baseType = (Class<T>) mockClassSetup.realClass;
      Class<T> baseClassType = baseType;
      Class<T> mockedClass = null;

      if (baseType.isInterface()) {
         mockedClass = new MockedImplementationClass<T>(mockClassSetup.mockUp).createImplementation(baseType);
         baseClassType = mockedClass;
      }

      if (baseClassType != Object.class) {
         redefineClass(baseClassType, baseType, null);
         mockClassSetup.validateThatAllMockMethodsWereApplied();
      }

      makeSureAllSubtypesAreModified(baseType, false, null);
      return mockedClass;
   }
}
